# frozen_string_literal: true

module Hyde
  module DSL
    # Methods shared by probes, preprocessors and filters.
    module CommonMethods
      # Stop execution and generate a boilerplate response with the given code
      # @param errorcode [Integer]
      # @param backtrace [Array(String), nil]
      # @raise [UncaughtThrowError] throws :finish to return back to Server
      def die(errorcode, backtrace: nil)
        throw :finish, [errorcode].append(
          *(@origin.properties["handle.#{errorcode}"] or
            @origin.properties["handle.default"]).call(
              errorcode,
              backtrace: backtrace
            )
        )
      end

      # Bounce request to the next handler
      # @raise [UncaughtThrowError] throws :break to get out of the callback
      def bounce
        throw :break
      end
    end
  end
end
