# frozen_string_literal: true

module Hyde
  # Shared DSL methods
  module DSL
    # Path (and subclasses) DSL constructors
    module PathConstructors
      # Append a Node child object to the list of children
      def register(obj)
        unless obj.is_a? Hyde::Node
          raise StandardError, "register accepts node children only"
        end

        @origin.children.append(obj)
      end

      # Create a new {Hyde::Path} object
      def path(path, &setup)
        # i don't know WHAT is wrong with this thing. it just is wrong.
        # @sg-ignore
        register(Hyde::Path.new(path, parent: @origin, &setup))
      end

      # Create a new {Hyde::Probe} object
      def probe(path, &_setup)
        register(Hyde::Probe.new(path, parent: @origin))
      end

      # Create a new {Hyde::GETHandler} object
      def get(path, &setup)
        register(Hyde::GETHandler.new(path, parent: @origin, &setup))
      end

      # create a new {Hyde::POSTHandler} object
      def post(path, &setup)
        register(Hyde::POSTHandler.new(path, parent: @origin, &setup))
      end

      # Create a new {Hyde::PUTHandler} object
      def put(path, &setup)
        register(Hyde::PUTHandler.new(path, parent: @origin, &setup))
      end

      # Create a new {Hyde::HEADHandler} object
      def head(path, &setup)
        register(Hyde::HEADHandler.new(path, parent: @origin, &setup))
      end

      # Create a new {Hyde::DELETEHandler} object
      def delete(path, &setup)
        register(Hyde::DELETEHandler.new(path, parent: @origin, &setup))
      end

      # Create a new {Hyde::CONNECTHandler} object
      def connect(path, &setup)
        register(Hyde::CONNECTHandler.new(path, parent: @origin, &setup))
      end

      # Create a new {Hyde::TRACEHandler} object
      def trace(path, &setup)
        register(Hyde::TRACEHandler.new(path, parent: @origin, &setup))
      end

      # Create a new {Hyde::PATCHHandler} object
      def patch(path, &setup)
        register(Hyde::PATCHHandler.new(path, parent: @origin, &setup))
      end

      # Create a new {Hyde::OPTIONSHandler} object
      def options(path, &setup)
        register(Hyde::OPTIONSHandler.new(path, parent: @origin, &setup))
      end

      # Create a new {Hyde::GETHandler} that serves static files
      def serve(path)
        register(Hyde::ServeHandler.new(path, parent: @origin))
      end
    end
  end
end
