# frozen_string_literal: true

module Hyde
  # Shared DSL methods
  module DSL
    # Common path methods
    module PathMethods
      # Set path index
      # @param index [Array,String]
      def index(index)
        case index
        when Array
          @origin.properties['index'] = index
        when String
          @origin.properties['index'] = [index]
        else
          raise StandardError, "index should be an Array or a String"
        end
      end

      # Set root path (appends matched part of the path).
      # @param path [String
      def root(path)
        @origin.root = path
      end

      # Set root path (without appending matched part).
      # @param path [String
      def remap(path)
        @origin.remap = path
      end

      # Add a preprocessor to the path.
      # Does not modify path execution.
      # @param block [#call]
      # @yieldparam request [Hyde::Request]
      def preprocess(&block)
        @origin.preprocess(&block)
        block
      end

      # Add a postprocessor to the path.
      # @param block [#call]
      # @yieldparam request [Hyde::Request]
      # @yieldparam response [Hyde::Response]
      def postprocess(&block)
        @origin.postprocess(&block)
        block
      end

      # Add a filter to the path.
      # Blocks path access if a filter returns false.
      # @param block [#call]
      # @yieldparam request [Hyde::Request]
      def filter(&block)
        @origin.filter(&block)
        block
      end
    end
  end
end
