# frozen_string_literal: true

require_relative 'node'
require_relative 'util/lookup'
require 'pp'

module Hyde
  autoload :Handler, "hyde/probe/handler"
  autoload :GETHandler, "hyde/probe/http_method"
  autoload :POSTHandler, "hyde/probe/http_method"
  autoload :HEADHandler, "hyde/probe/http_method"
  autoload :PUTHandler, "hyde/probe/http_method"
  autoload :DELETEHandler, "hyde/probe/http_method"
  autoload :CONNECTHandler, "hyde/probe/http_method"
  autoload :OPTIONSHandler, "hyde/probe/http_method"
  autoload :TRACEHandler, "hyde/probe/http_method"
  autoload :PATCHHandler, "hyde/probe/http_method"
  autoload :ServeHandler, "hyde/probe/serve_handler"

  # Binding that provides execution context for Probes.
  class ProbeBinding
    include Hyde::DSL::ProbeMethods
    include Hyde::DSL::CommonMethods

    def initialize(origin)
      @origin = origin
    end
  end

  # Test probe. Also base for all "reactive" nodes.
  class Probe < Hyde::Node
    # @param path [Object]
    # @param parent [Hyde::Node]
    def initialize(path, parent:)
      super(path, parent: parent)
    end

    attr_reader :properties

    # Method callback on successful request navigation.
    # Throws an error upon reaching the path.
    # This behaviour should only be used internally.
    # @param request [Hyde::Request]
    # @return [Boolean] true if further navigation is possible
    # @raise [StandardError]
    def process(request)
      return reject(request) unless request.path.match?(/^\/?$/)

      raise StandardError, <<~STREND
        probe reached #{request.splat.inspect}, #{request.param.inspect}
        #{request.pretty_inspect}
      STREND
    end
  end
end
