# frozen_string_literal: true

module Hyde
  # Probe that executes callback on a GET
  class GETHandler < Hyde::Handler
    METHOD = "GET"

    # Method callback on successful request navigation.
    # Runs block supplied with object initialization.
    # Request's #splat and #param are passed to block.
    #
    # Callback's returned should be one of viable responses:
    #
    # - {Hyde::Response} object
    # - An array that matches Rack return form
    # - An array that matches old (Rack 2.x) return form
    # - A string (returned as HTML with code 200)
    # - false (bounces the request to next handler)
    # @param request [Hyde::Request]
    # @return [Boolean] true if further navigation is possible
    # @raise [UncaughtThrowError] may raise if die() is called.
    def process(request)
      unless request.request_method.casecmp(self.class::METHOD).zero?
        return false
      end

      super(request)
    end
  end

  # Probe that executes callback on a POST
  class POSTHandler < GETHandler
    METHOD = "POST"
  end

  # Probe that executes callback on a HEAD
  class HEADHandler < GETHandler
    METHOD = "HEAD"
  end

  # Probe that executes callback on a PUT
  class PUTHandler < GETHandler
    METHOD = "PUT"
  end

  # Probe that executes callback on a DELETE
  class DELETEHandler < GETHandler
    METHOD = "DELETE"
  end

  # Probe that executes callback on a CONNECT
  class CONNECTHandler < GETHandler
    METHOD = "CONNECT"
  end

  # Probe that executes callback on a OPTIONS
  class OPTIONSHandler < GETHandler
    METHOD = "OPTIONS"
  end

  # Probe that executes callback on a TRACE
  class TRACEHandler < GETHandler
    METHOD = "TRACE"
  end

  # Probe that executes callback on a PATCH
  class PATCHHandler < GETHandler
    METHOD = "PATCH"
  end
end
