# frozen_string_literal: true

module Hyde
  # Probe that sends files from a location
  class ServeHandler < Hyde::Probe
    # @param path [Object]
    # @param parent [Hyde::Node]
    # @param exec [#call]
    def initialize(path, parent:)
      super(path, parent: parent)
    end

    attr_accessor :response

    # Method callback on successful request navigation.
    # Tries to serve files matched by handler
    # @param request [Hyde::Request]
    # @return [Boolean] true if file was found
    def process(request)
      path = File.expand_path(request.filepath)
      return unless path.start_with? @properties["path"]

      File.open(path.delete_suffix("/"))
    rescue StandardError
      false
    end
  end
end
