# frozen_string_literal: true

require_relative 'path'
require_relative 'request'
require_relative 'dsl/path_constructors'
require_relative 'util/html'

module Hyde
  class ServerBinding < Hyde::PathBinding
  end

  # A specialized path that can be used directly as a Rack application.
  class Server < Hyde::Path
    Binding = ServerBinding

    # @param parent [Hyde::Node, nil] Parent object to inherit properties to
    # @param setup [#call] Setup block
    def initialize(parent: nil, &setup)
      super("", parent: parent, &setup)
      return if parent

      {
        "index" => [],
        "handle.default" => proc do |code, backtrace: nil|
          page = Hyde::Util.default_error_page(code, backtrace)
          headers = {
            "content-length": page.length,
            "content-type": "text/html"
          }
          [headers, page]
        end,
        "path" => "/"
      }.each { |k, v| @properties[k] = v unless @properties[k] }
    end

    # Rack ingress point.
    # This should not be called under any circumstances twice in the same application,
    # although server nesting for the purpose of creating virtual hosts is allowed.
    # @param env [Hash]
    # @return [Array(Integer,Hash,Array)]
    def call(env)
      request = Hyde::Request.new(env)
      response = catch(:finish) do
        go(request)
      end
      request.run_postprocessors(response)
      Response.convert(response).finalize
    end
  end
end
