# frozen_string_literal: true

module Hyde
  # Adapter class for template engines
  # @abstract This class does not actually do anything template related.
  class Template
    # @param filename [String]
    # @param context [Binding, nil]
    # @param locals [Hash{Symbol => Object}, nil]
    def initialize(filename, context: nil, locals: nil)
      @data = File.read(filename)
      @context = context or binding
      @locals.each do |k, v|
        @context.local_variable_set(k, v)
      end
    end

    # Prepare template before rendering.
    # (This method is an interface stub)
    def prepare
      @template = nil # your template generator goes here
    end

    # Render template
    # (This method is an interface stub)
    def render
      (@template ||= prepare).render(@context)
    end
  end
end
