# frozen_string_literal: true

require 'uri'

module Hyde
  module Util
    # Parse parametrized header values
    # @param input [String]
    # @param sep [String, Regexp]
    # @return [Array(String, Hash)]
    def self.parse_value(input, sep = ";")
      parts = input.split(sep).map { |x| URI.decode_uri_component(x).strip }
      base = parts.shift
      opts = parts.map do |raw|
        key, value = raw.match(/^([^=]*)(?:=(.*)|)\Z/).to_a[1..]
        [key, ((value&.match?(/^".*"$/) ? value[1..-2] : value) or true)]
      end.to_h
      [base, opts]
    end

    # Construct a parametrized header value
    # @param input [String]
    # @param opts [Hash]
    # @return [String]
    def self.make_value(input, opts, sep = ";")
      unless input.match?(/^[\w!#$%&'*+-.^_`|~]*=?[^[:cntrl:]\\",;]+$/)
        raise StandardError, "input contains invalid characters"
      end

      output = input
      opts.each do |key, value|
        output += if value.is_a? String
                    "#{sep} #{key}=#{value}"
                  else
                    "#{sep} #{key}"
                  end
      end
      output
    end
  end
end
