# frozen_string_literal: true

require_relative "../lib/hyde/pattern_matching"
require "test/unit"

class TestGlob < Test::Unit::TestCase
  include Hyde::PatternMatching
  # match? test
  def test_matchq
    puts "Test group: *"
    unit = Glob.new("/test/*")
    [
      "est/anything", false,
      "/test", false,
      "/test/anything", true,
      "/test//as", true,
      "/test/", false,
      "/test/as/whatever", true,
      "test/as", true
    ].each_slice(2) do |test, result|
      puts("Testing: #{test}")
      assert_equal(result, unit.match?(test))
    end
    unit = Glob.new("/test/*/something")
    [
      "/test/s/something", true,
      "/test//something", false,
      "/test/something", false,
      "test/b/something", true,
      "/test/b/someth", false
    ].each_slice(2) do |test, result|
      puts("Testing: #{test}")
      assert_equal(result, unit.match?(test))
    end
    puts "Test group: **"
    unit = Glob.new("/test/**/something")
    [
      "/test/s/something", true,
      "/test/dir/something", true,
      "/test/something", true,
      "test/b/something", true,
      "/test/b/someth", false,
      "/test/a/b/c/something", true,
      "/test/a/b/csomething", false,
      "/testsomething", false,
      "/something", false
    ].each_slice(2) do |test, result|
      puts("Testing: #{test}")
      assert_equal(result, unit.match?(test))
    end
    unit = Glob.new("/test/**/*.php")
    [
      "/test/archive.php", true,
      "/test/assets/thing.js", false,
      "/test/assetsthing.js", false,
      "/test/parts/thing.php", true,
      "/test/partsthing.php", true,
      "/test/.php", true,
      "/test/parts/extra/test.php", true,
      "test/archive.php", true,
      "test/assets/thing.js", false,
      "test/assetsthing.js", false,
      "test/parts/thing.php", true,
      "test/partsthing.php", true,
      "test/.php", true,
      "test/parts/extra/test.php", true,
      "/test/parts/extra/test.php/literally/anything/here", true
    ].each_slice(2) do |test, result|
      puts("Testing: #{test}")
      assert_equal(result, unit.match?(test))
    end
    unit = Glob.new("**/*.php")
    [
      "archive.php", true,
      "assets/thing.js", false,
      "assetsthing.js", false,
      "parts/thing.php", true,
      "partsthing.php", true,
      ".php", true,
      "parts/extra/test.php", true,
      "archive.php", true,
      "assets/thing.js", false,
      "assetsthing.js", false,
      "parts/thing.php", true,
      "partsthing.php", true,
      ".php", true,
      "parts/extra/test.php", true,
      "parts/extra/test.php/literally/anything/here", true
    ].each_slice(2) do |test, result|
      puts("Testing: #{test}")
      assert_equal(result, unit.match?(test))
    end
    puts "Test group: char ranges"
    unit = Glob.new("/test/[9-z]+")
    [
      "/test/t+", true,
      "/test/$+", false,
      "/test/aosidujqwi", true,
      "/test/9+", true
    ].each_slice(2) do |test, result|
      puts("Testing: #{test}")
      assert_equal(result, unit.match?(test))
    end
    puts "Test group: named captures"
    unit = Glob.new("/test/:name/something")
    [
      "/test/something/something", true,
      "/test/asd/something/extra", true,
      "/test//something/what", false,
      "/test/something/what", false,
      "/test/asd/asd/something/what", false
    ].each_slice(2) do |test, result|
      puts("Testing: #{test}")
      assert_equal(result, unit.match?(test))
    end
    puts "Test group: alternator groups"
    unit = Glob.new("/test/(one|another-thing)/something")
    [
      "/test/one/something", true,
      "/test/another-thing/something", true,
      "/test/one/something/more", true,
      "test/another-thing/something/even/more", true,
      "/test/two/something", false,
      "test//something", false,
      "/test/something", false
    ].each_slice(2) do |test, result|
      puts("Testing: #{test}")
      assert_equal(result, unit.match?(test))
    end
  end

  def test_match
    puts "Test group: *"
    unit = Glob.new("/test/*/something")
    [
      "/test/include/something", ["", ["include"], {}],
      "/test//something", false,
      "/test/include/something/extra", ["/extra", ["include"], {}],
      "/test/shlurpglurp/asdsdsad/something", false
    ].each_slice(2) do |test, result|
      puts("Testing: #{test}")
      assert_equal(result, unit.match(test))
    end
    puts "Test group: **"
    unit = Glob.new("/test/**/something")
    [
      "/test/include/something", ["", ["include"], {}],
      "/test/something", ["", [""], {}],
      "/testsomething", false,
      "/test/dir/subdir/something", ["", ["dir/subdir"], {}],
      "/test/dir/subdir/something/extra", ["/extra", ["dir/subdir"], {}]
    ].each_slice(2) do |test, result|
      puts("Testing: #{test}")
      assert_equal(result, unit.match(test))
    end
    puts "Test group: **/*.ext"
    unit = Glob.new("/test/**/*.ext")
    [
      "/test/file.ext", ["", ["", "file"], {}],
      "/test/path/file.ext", ["", ["path", "file"], {}],
      "/test/path/file.php", false,
      "/test/path/.ext", ["", ["path", ""], {}],
      "/test/dir/subdir/file.ext", ["", ["dir/subdir", "file"], {}]
    ].each_slice(2) do |test, result|
      puts("Testing: #{test}")
      assert_equal(result, unit.match(test))
    end
    puts "Test group: named captures"
    assert_equal(false, Glob.can_convert?("/test/as:something/thing"))
    assert_equal(false, Glob.can_convert?("/test/:&(&(@42/thing"))
    unit = Glob.new("/test/:name/thing")
    [
      "/test/thing", false,
      "/test//thing", false,
      "/test/path/thing", ["", [], { name: "path" }],
      "/test/peepee*(^*&^*&68123/thing", ["", [], { name: "peepee*(^*&^*&68123" }],
      "/test/dum/file.ext", false
    ].each_slice(2) do |test, result|
      puts("Testing: #{test}")
      assert_equal(result, unit.match(test))
    end
    puts "Test group: alternator groups"
    unit = Glob.new("/test/(this|that|anything_like-this)/thing")
    [
      "/test/thing", false,
      "/test//thing", false,
      "/test/this/thing", ["", ["this"], {}],
      "/test/that/thing", ["", ["that"], {}],
      "/test/anything_likeeeeethis/thing", false,
      "/test/anything_like-this/thing", ["", ["anything_like-this"], {}]
    ].each_slice(2) do |test, result|
      puts("Testing: #{test}")
      assert_equal(result, unit.match(test))
    end
    #i hate writing unit tests
  end
end
